@@ -9,10 +9,12 @@
 from __future__ import absolute_import
 
 from .components import Worker, Customer, Collector, Task
-from .exceptions import ZeronimoException, WorkerNotFound
+from .exceptions import (
+    ZeronimoException, WorkerNotFound, SocketClosed, UnexpectedMessage, raises)
 from .messaging import send, recv
 
 
-__version__ = '0.1.0'
+__version__ = '0.1.1'
 __all__ = ['Worker', 'Customer', 'Collector', 'Task', 'ZeronimoException',
-           'WorkerNotFound', 'send', 'recv']
+           'WorkerNotFound', 'SocketClosed', 'UnexpectedMessage', 'raises',
+           'send', 'recv']
