@@ -8,27 +8,30 @@
 """
 from __future__ import absolute_import
 from collections import Iterable, Mapping, Sequence, Set
+from contextlib import contextmanager
 import functools
 from types import MethodType
+import warnings
 
-from gevent import GreenletExit, spawn, Timeout
+from gevent import Greenlet, GreenletExit, Timeout
 from gevent.queue import Empty, Queue
 try:
     from libuuid import uuid4_bytes
 except ImportError:
     import uuid
     uuid4_bytes = lambda: uuid.uuid4().get_bytes()
-from msgpack import ExtraData
+from msgpack import ExtraData, UnpackValueError
 import zmq.green as zmq
 
-from .exceptions import make_worker_not_found
+from .exceptions import (
+    SocketClosed, UnexpectedMessage, make_worker_not_found)
 from .helpers import cls_name, make_repr
 from .messaging import (
     ACK, DONE, ACCEPT, REJECT, RETURN, RAISE, YIELD, BREAK,
     Call, Reply, send, recv)
 
 
-__all__ = ['Runnable', 'Worker', 'Customer', 'Collector', 'Task']
+__all__ = ['Component', 'Worker', 'Customer', 'Collector', 'Task']
 
 
 def is_iterator(obj):
@@ -36,13 +39,15 @@ def is_iterator(obj):
     return (isinstance(obj, Iterable) and not isinstance(obj, serializable))
 
 
-class Runnable(object):
-    """A runnable object should implement :meth:`run`. :attr:`running` is
-    ensured to be ``True`` while :meth:`run` is runnig.
+class Component(object):
+    """A component should implement :meth:`run`. :attr:`running` is ensured to
+    be ``True`` while :meth:`run` is executing.
     """
 
+    greenlet_class = Greenlet
+
     def __new__(cls, *args, **kwargs):
-        obj = super(Runnable, cls).__new__(cls)
+        obj = super(Component, cls).__new__(cls)
         cls._patch(obj)
         return obj
 
@@ -69,7 +74,7 @@ class Runnable(object):
     def start(self):
         if self.is_running():
             raise RuntimeError('{0} already running'.format(cls_name(self)))
-        self._running = spawn(self._inner_run)
+        self._running = self.greenlet_class.spawn(self._inner_run)
         self._running.join(0)
         return self._running
 
@@ -89,7 +94,7 @@ class Runnable(object):
         return self._running is not None
 
 
-class Worker(Runnable):
+class Worker(Component):
     """The worker object runs an RPC service of an object through ZMQ sockets.
     The ZMQ sockets should be PULL or SUB socket type. The PULL sockets receive
     Round-robin calls; the SUB sockets receive Publish-subscribe
@@ -116,8 +121,9 @@ class Worker(Runnable):
     def __init__(self, obj, sockets, info=None):
         super(Worker, self).__init__()
         self.obj = obj
-        if set(s.socket_type for s in sockets).difference([zmq.PULL, zmq.SUB]):
-            raise ValueError('Worker socket should be PULL or SUB')
+        socket_types = set(s.socket_type for s in sockets)
+        if socket_types.difference([zmq.PULL, zmq.SUB, zmq.PAIR]):
+            raise ValueError('Worker wraps PULL or SUB or PAIR sockets')
         self.sockets = sockets
         self.info = info
         self._cached_reply_sockets = {}
@@ -144,46 +150,67 @@ class Worker(Runnable):
                 assert event & zmq.POLLIN
                 try:
                     call = Call(*recv(socket))
-                except (TypeError, ExtraData):
-                    # TODO: warning
+                except (TypeError, ExtraData, UnpackValueError) as exc:
+                    warning = UnexpectedMessage(
+                        'Received unexpected message: {!r}'.format(exc.serial))
+                    warning.serial = exc.serial
+                    warnings.warn(warning)
                     continue
-                spawn(self.work, call, socket.context)
+                self.greenlet_class.spawn(self.work, socket, call)
 
-    def work(self, call, context):
-        """Invokes a function and send results to the customer. It supports
+    def work(self, socket, call):
+        """Calls a function and send results to the collector. It supports
         all of function actions. A function could return, yield, raise any
-        picklable objects.
+        packable objects.
         """
         work_id = uuid4_bytes()
-        socket = None
-        if call.collector_address is not None:
-            socket = self.get_reply_socket(call.collector_address, context)
+        socket = self.get_reply_socket(socket, call.collector_address)
+        channel = (None, None)
+        if socket is not None:
             channel = (call.call_id, work_id)
             method = ACCEPT if self.accepting else REJECT
             self.send_reply(socket, method, self.info, *channel)
         if not self.accepting:
             return
-        try:
+        with self.exception_sending(socket, *channel) as raised:
             val = self.call(call)
-        except Exception as error:
-            socket and self.send_reply(socket, RAISE, error, *channel)
-            raise
+        if raised():
+            return
         if is_iterator(val):
             vals = val
-            try:
+            with self.exception_sending(socket, *channel):
                 for val in vals:
                     socket and self.send_reply(socket, YIELD, val, *channel)
                 socket and self.send_reply(socket, BREAK, None, *channel)
-            except Exception as error:
-                socket and self.send_reply(socket, RAISE, error, *channel)
-                raise
         else:
             socket and self.send_reply(socket, RETURN, val, *channel)
 
+    @contextmanager
+    def exception_sending(self, socket, *channel):
+        """Sends an exception which occurs in the context to the collector.
+        It raises the caught exception if that was not expected.
+        :func:`zeronimo.exceptions.raises` declares expected exception types.
+        """
+        raised = []
+        try:
+            yield lambda: bool(raised)
+        except BaseException as exc:
+            raised.append(True)
+            socket and self.send_reply(socket, RAISE, exc, *channel)
+            # don't raise at worker-side if the exception was expected
+            if not getattr(exc, '_znm_expected', False):
+                raise
+
     def call(self, call):
+        """Calls a function."""
         return getattr(self.obj, call.function_name)(*call.args, **call.kwargs)
 
-    def get_reply_socket(self, address, context):
+    def get_reply_socket(self, socket, address):
+        if socket.type == zmq.PAIR:
+            return socket
+        if address is None:
+            return
+        context = socket.context
         try:
             sockets = self._cached_reply_sockets[context]
         except KeyError:
@@ -201,7 +228,7 @@ class Worker(Runnable):
         reply = Reply(method, data, call_id, work_id)
         try:
             return send(socket, reply, zmq.NOBLOCK)
-        except zmq.Again:
+        except (zmq.Again, zmq.ZMQError):
             pass
 
     def __repr__(self):
@@ -216,14 +243,14 @@ class Customer(object):
     _znm_topic = None
 
     def __init__(self, socket, collector=None):
-        if socket.type not in (zmq.PUSH, zmq.PUB):
-            raise ValueError('Customer socket should be PUSH or PUB')
+        if socket.type not in [zmq.PUSH, zmq.PUB, zmq.PAIR]:
+            raise ValueError('Customer wraps PUSH or PUB or PAIR socket')
         self._znm_socket = socket
         self._znm_collector = collector
 
     def __getitem__(self, topic):
         if self._znm_socket.type != zmq.PUB:
-            raise ValueError('Customer socket should be PUB to set a topic')
+            raise ValueError('Only customer with PUB socket could set a topic')
         cls = type(self)
         customer = cls(self._znm_socket, self._znm_collector)
         customer._znm_topic = topic
@@ -263,11 +290,15 @@ class Customer(object):
         return tasks if is_fanout else tasks[0]
 
 
-class Collector(Runnable):
+class Collector(Component):
 
-    def __init__(self, socket, address, as_task=False, timeout=0.01):
-        if socket.type != zmq.PULL:
-            raise ValueError('Collector socket should be PULL')
+    def __init__(self, socket, address=None, as_task=False, timeout=0.01):
+        if socket.type not in [zmq.PULL, zmq.PAIR]:
+            raise ValueError('Collector wraps PULL or PAIR socket')
+        if address is None and socket.type != zmq.PAIR:
+            raise ValueError('Address required')
+        if address is not None and socket.type == zmq.PAIR:
+            raise ValueError('Address not required when using PAIR socket')
         self.socket = socket
         self.address = address
         self.as_task = as_task
@@ -282,12 +313,21 @@ class Collector(Runnable):
             except (TypeError, ExtraData):
                 # TODO: warning
                 continue
+            except zmq.ZMQError:
+                exc = SocketClosed('Collector socket closed')
+                self.put_all(Reply(RAISE, exc, None, None))
+                break
             try:
                 self.dispatch_reply(reply)
             except KeyError:
                 # TODO: warning
                 continue
 
+    def put_all(self, reply):
+        for reply_queues in self.reply_queues.itervalues():
+            for reply_queue in reply_queues.itervalues():
+                reply_queue.put(reply)
+
     def dispatch_reply(self, reply):
         if reply.method & ACK:
             self.reply_queues[reply.call_id][None].put(reply)
@@ -350,8 +390,8 @@ class Collector(Runnable):
         except KeyError:
             pass
         else:
-            spawn(self._collect_more_tasks,
-                  tasks, ack_queue.get, call_id, limit)
+            self.greenlet_class.spawn(
+                self._collect_more_tasks, tasks, ack_queue.get, call_id, limit)
         return tasks
 
     def _collect_more_tasks(self, tasks, get_reply, call_id, limit=None):
@@ -388,6 +428,7 @@ class Collector(Runnable):
         del reply_queues[task.work_id]
         if not reply_queues:
             del self.reply_queues[task.call_id]
+            self.missing_queues.pop(task.call_id, None)
 
 
 class Task(object):
@@ -411,7 +452,7 @@ class Task(object):
         """Gets the result."""
         reply = self.reply_queue.get()
         assert not reply.method & ACK
-        if reply.method == DONE:
+        if reply.method & DONE:
             self.collector.task_done(self)
         if reply.method == RETURN:
             return reply.data
